@@ -4,23 +4,20 @@ from starlette.datastructures import URLPath
 from starlette.exceptions import ExceptionMiddleware
 from starlette.middleware.base import BaseHTTPMiddleware
 from starlette.middleware.errors import ServerErrorMiddleware
-from starlette.middleware.lifespan import LifespanMiddleware
 from starlette.routing import BaseRoute, Router
-from starlette.schemas import BaseSchemaGenerator
-from starlette.types import ASGIApp, ASGIInstance, Scope
+from starlette.types import ASGIApp, Receive, Scope, Send
 
 
 class Starlette:
-    def __init__(self, debug: bool = False, template_directory: str = None) -> None:
+    def __init__(
+        self, debug: bool = False, routes: typing.List[BaseRoute] = None
+    ) -> None:
         self._debug = debug
-        self.router = Router()
+        self.router = Router(routes)
         self.exception_middleware = ExceptionMiddleware(self.router, debug=debug)
         self.error_middleware = ServerErrorMiddleware(
             self.exception_middleware, debug=debug
         )
-        self.lifespan_middleware = LifespanMiddleware(self.error_middleware)
-        self.schema_generator = None  # type: typing.Optional[BaseSchemaGenerator]
-        self.template_env = self.load_template_env(template_directory)
 
     @property
     def routes(self) -> typing.List[BaseRoute]:
@@ -36,33 +33,8 @@ class Starlette:
         self.exception_middleware.debug = value
         self.error_middleware.debug = value
 
-    def load_template_env(self, template_directory: str = None) -> typing.Any:
-        if template_directory is None:
-            return None
-
-        # Import jinja2 lazily.
-        import jinja2
-
-        @jinja2.contextfunction
-        def url_for(context: dict, name: str, **path_params: typing.Any) -> str:
-            request = context["request"]
-            return request.url_for(name, **path_params)
-
-        loader = jinja2.FileSystemLoader(str(template_directory))
-        env = jinja2.Environment(loader=loader, autoescape=True)
-        env.globals["url_for"] = url_for
-        return env
-
-    def get_template(self, name: str) -> typing.Any:
-        return self.template_env.get_template(name)
-
-    @property
-    def schema(self) -> dict:
-        assert self.schema_generator is not None
-        return self.schema_generator.get_schema(self.routes)
-
     def on_event(self, event_type: str) -> typing.Callable:
-        return self.lifespan_middleware.on_event(event_type)
+        return self.router.lifespan.on_event(event_type)
 
     def mount(self, path: str, app: ASGIApp, name: str = None) -> None:
         self.router.mount(path, app=app, name=name)
@@ -88,7 +60,7 @@ class Starlette:
             )
 
     def add_event_handler(self, event_type: str, func: typing.Callable) -> None:
-        self.lifespan_middleware.add_event_handler(event_type, func)
+        self.router.lifespan.add_event_handler(event_type, func)
 
     def add_route(
         self,
@@ -156,6 +128,6 @@ class Starlette:
     def url_path_for(self, name: str, **path_params: str) -> URLPath:
         return self.router.url_path_for(name, **path_params)
 
-    def __call__(self, scope: Scope) -> ASGIInstance:
+    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
         scope["app"] = self
-        return self.lifespan_middleware(scope)
+        await self.error_middleware(scope, receive, send)
