@@ -31,6 +31,13 @@ class MultiPartMessage(Enum):
     END = 8
 
 
+def _user_safe_decode(src: bytes, codec: str) -> str:
+    try:
+        return src.decode(codec)
+    except (UnicodeDecodeError, LookupError):
+        return src.decode("latin-1")
+
+
 class FormParser:
     def __init__(
         self, headers: Headers, stream: typing.AsyncGenerator[bytes, None]
@@ -153,6 +160,9 @@ class MultiPartParser:
     async def parse(self) -> FormData:
         # Parse the Content-Type header to get the multipart boundary.
         content_type, params = parse_options_header(self.headers["Content-Type"])
+        charset = params.get(b"charset", "utf-8")
+        if type(charset) == bytes:
+            charset = charset.decode("latin-1")
         boundary = params.get(b"boundary")
 
         # Callbacks dictionary.
@@ -171,7 +181,8 @@ class MultiPartParser:
         parser = multipart.MultipartParser(boundary, callbacks)
         header_field = b""
         header_value = b""
-        raw_headers = []  # type: typing.List[typing.Tuple[bytes, bytes]]
+        content_disposition = None
+        content_type = b""
         field_name = ""
         data = b""
         file = None  # type: typing.Optional[UploadFile]
@@ -187,24 +198,30 @@ class MultiPartParser:
             self.messages.clear()
             for message_type, message_bytes in messages:
                 if message_type == MultiPartMessage.PART_BEGIN:
-                    raw_headers = []
+                    content_disposition = None
+                    content_type = b""
                     data = b""
                 elif message_type == MultiPartMessage.HEADER_FIELD:
                     header_field += message_bytes
                 elif message_type == MultiPartMessage.HEADER_VALUE:
                     header_value += message_bytes
                 elif message_type == MultiPartMessage.HEADER_END:
-                    raw_headers.append((header_field.lower(), header_value))
+                    field = header_field.lower()
+                    if field == b"content-disposition":
+                        content_disposition = header_value
+                    elif field == b"content-type":
+                        content_type = header_value
                     header_field = b""
                     header_value = b""
                 elif message_type == MultiPartMessage.HEADERS_FINISHED:
-                    headers = Headers(raw=raw_headers)
-                    content_disposition = headers.get("Content-Disposition")
                     disposition, options = parse_options_header(content_disposition)
-                    field_name = options[b"name"].decode("latin-1")
+                    field_name = _user_safe_decode(options[b"name"], charset)
                     if b"filename" in options:
-                        filename = options[b"filename"].decode("latin-1")
-                        file = UploadFile(filename=filename)
+                        filename = _user_safe_decode(options[b"filename"], charset)
+                        file = UploadFile(
+                            filename=filename,
+                            content_type=content_type.decode("latin-1"),
+                        )
                     else:
                         file = None
                 elif message_type == MultiPartMessage.PART_DATA:
@@ -214,7 +231,7 @@ class MultiPartParser:
                         await file.write(message_bytes)
                 elif message_type == MultiPartMessage.PART_END:
                     if file is None:
-                        items.append((field_name, data.decode("latin-1")))
+                        items.append((field_name, _user_safe_decode(data, charset)))
                     else:
                         await file.seek(0)
                         items.append((field_name, file))
