@@ -1,4 +1,3 @@
-import functools
 import json
 import typing
 
@@ -7,7 +6,7 @@ from starlette.background import BackgroundTasks
 from starlette.concurrency import run_in_threadpool
 from starlette.requests import Request
 from starlette.responses import HTMLResponse, JSONResponse, PlainTextResponse, Response
-from starlette.types import ASGIInstance, Receive, Scope, Send
+from starlette.types import Receive, Scope, Send
 
 try:
     import graphene
@@ -27,8 +26,10 @@ class GraphQLApp:
         schema: "graphene.Schema",
         executor: typing.Any = None,
         executor_class: type = None,
+        graphiql: bool = True,
     ) -> None:
         self.schema = schema
+        self.graphiql = graphiql
         if executor is None:
             # New style in 0.10.0. Use 'executor_class'.
             # See issue https://github.com/encode/starlette/issues/242
@@ -44,20 +45,21 @@ class GraphQLApp:
             self.executor_class = None
             self.is_async = isinstance(executor, AsyncioExecutor)
 
-    def __call__(self, scope: Scope) -> ASGIInstance:
-        return functools.partial(self.asgi, scope=scope)
-
-    async def asgi(self, receive: Receive, send: Send, scope: Scope) -> None:
+    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
         if self.executor is None and self.executor_class is not None:
             self.executor = self.executor_class()
 
         request = Request(scope, receive=receive)
         response = await self.handle_graphql(request)
-        await response(receive, send)
+        await response(scope, receive, send)
 
     async def handle_graphql(self, request: Request) -> Response:
         if request.method in ("GET", "HEAD"):
             if "text/html" in request.headers.get("Accept", ""):
+                if not self.graphiql:
+                    return PlainTextResponse(
+                        "Not Found", status_code=status.HTTP_404_NOT_FOUND
+                    )
                 return await self.handle_graphiql(request)
 
             data = request.query_params  # type: typing.Mapping[str, typing.Any]
