@@ -3,8 +3,8 @@ import re
 import typing
 
 from starlette.datastructures import Headers, MutableHeaders
-from starlette.responses import PlainTextResponse
-from starlette.types import ASGIApp, ASGIInstance, Message, Receive, Scope, Send
+from starlette.responses import PlainTextResponse, Response
+from starlette.types import ASGIApp, Message, Receive, Scope, Send
 
 ALL_METHODS = ("DELETE", "GET", "OPTIONS", "PATCH", "POST", "PUT")
 
@@ -56,28 +56,32 @@ class CORSMiddleware:
         self.app = app
         self.allow_origins = allow_origins
         self.allow_methods = allow_methods
-        self.allow_headers = allow_headers
+        self.allow_headers = [h.lower() for h in allow_headers]
         self.allow_all_origins = "*" in allow_origins
         self.allow_all_headers = "*" in allow_headers
         self.allow_origin_regex = compiled_allow_origin_regex
         self.simple_headers = simple_headers
         self.preflight_headers = preflight_headers
 
-    def __call__(self, scope: Scope) -> ASGIInstance:
-        if scope["type"] == "http":
-            method = scope["method"]
-            headers = Headers(scope=scope)
-            origin = headers.get("origin")
+    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
+        if scope["type"] != "http":  # pragma: no cover
+            await self.app(scope, receive, send)
+            return
 
-            if origin is not None:
-                if method == "OPTIONS" and "access-control-request-method" in headers:
-                    return self.preflight_response(request_headers=headers)
-                else:
-                    return functools.partial(
-                        self.simple_response, scope=scope, request_headers=headers
-                    )
+        method = scope["method"]
+        headers = Headers(scope=scope)
+        origin = headers.get("origin")
 
-        return self.app(scope)
+        if origin is None:
+            await self.app(scope, receive, send)
+            return
+
+        if method == "OPTIONS" and "access-control-request-method" in headers:
+            response = self.preflight_response(request_headers=headers)
+            await response(scope, receive, send)
+            return
+
+        await self.simple_response(scope, receive, send, request_headers=headers)
 
     def is_allowed_origin(self, origin: str) -> bool:
         if self.allow_all_origins:
@@ -90,11 +94,10 @@ class CORSMiddleware:
 
         return origin in self.allow_origins
 
-    def preflight_response(self, request_headers: Headers) -> ASGIInstance:
+    def preflight_response(self, request_headers: Headers) -> Response:
         requested_origin = request_headers["origin"]
         requested_method = request_headers["access-control-request-method"]
         requested_headers = request_headers.get("access-control-request-headers")
-        requested_cookie = "cookie" in request_headers
 
         headers = dict(self.preflight_headers)
         failures = []
@@ -117,7 +120,7 @@ class CORSMiddleware:
         if self.allow_all_headers and requested_headers is not None:
             headers["Access-Control-Allow-Headers"] = requested_headers
         elif requested_headers is not None:
-            for header in requested_headers.split(","):
+            for header in [h.lower() for h in requested_headers.split(",")]:
                 if header.strip() not in self.allow_headers:
                     failures.append("headers")
 
@@ -131,11 +134,10 @@ class CORSMiddleware:
         return PlainTextResponse("OK", status_code=200, headers=headers)
 
     async def simple_response(
-        self, receive: Receive, send: Send, scope: Scope, request_headers: Headers
+        self, scope: Scope, receive: Receive, send: Send, request_headers: Headers
     ) -> None:
-        inner = self.app(scope)
         send = functools.partial(self.send, send=send, request_headers=request_headers)
-        await inner(receive, send)
+        await self.app(scope, receive, send)
 
     async def send(
         self, message: Message, send: Send, request_headers: Headers
@@ -146,18 +148,18 @@ class CORSMiddleware:
 
         message.setdefault("headers", [])
         headers = MutableHeaders(scope=message)
+        headers.update(self.simple_headers)
         origin = request_headers["Origin"]
         has_cookie = "cookie" in request_headers
 
         # If request includes any cookie headers, then we must respond
         # with the specific origin instead of '*'.
         if self.allow_all_origins and has_cookie:
-            self.simple_headers["Access-Control-Allow-Origin"] = origin
+            headers["Access-Control-Allow-Origin"] = origin
 
         # If we only allow specific origins, then we have to mirror back
         # the Origin header in the response.
         elif not self.allow_all_origins and self.is_allowed_origin(origin=origin):
             headers["Access-Control-Allow-Origin"] = origin
             headers.add_vary_header("Origin")
-        headers.update(self.simple_headers)
         await send(message)
