@@ -1,5 +1,6 @@
 import hashlib
 import http.cookies
+import inspect
 import json
 import os
 import stat
@@ -9,8 +10,9 @@ from mimetypes import guess_type
 from urllib.parse import quote_plus
 
 from starlette.background import BackgroundTask
+from starlette.concurrency import iterate_in_threadpool
 from starlette.datastructures import URL, MutableHeaders
-from starlette.types import Receive, Send
+from starlette.types import Receive, Scope, Send
 
 try:
     import aiofiles
@@ -37,10 +39,7 @@ class Response:
         media_type: str = None,
         background: BackgroundTask = None,
     ) -> None:
-        if content is None:
-            self.body = b""
-        else:
-            self.body = self.render(content)
+        self.body = self.render(content)
         self.status_code = status_code
         if media_type is not None:
             self.media_type = media_type
@@ -48,6 +47,8 @@ class Response:
         self.init_headers(headers)
 
     def render(self, content: typing.Any) -> bytes:
+        if content is None:
+            return b""
         if isinstance(content, bytes):
             return content
         return content.encode(self.charset)
@@ -116,7 +117,7 @@ class Response:
     def delete_cookie(self, key: str, path: str = "/", domain: str = None) -> None:
         self.set_cookie(key, expires=0, max_age=0, path=path, domain=domain)
 
-    async def __call__(self, receive: Receive, send: Send) -> None:
+    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
         await send(
             {
                 "type": "http.response.start",
@@ -138,39 +139,6 @@ class PlainTextResponse(Response):
     media_type = "text/plain"
 
 
-class TemplateResponse(Response):
-    media_type = "text/html"
-
-    def __init__(
-        self,
-        template: typing.Any,
-        context: dict,
-        status_code: int = 200,
-        headers: dict = None,
-        media_type: str = None,
-        background: BackgroundTask = None,
-    ):
-        if "request" not in context:
-            raise ValueError('context must include a "request" key')
-        self.template = template
-        self.context = context
-        content = template.render(context)
-        super().__init__(content, status_code, headers, media_type, background)
-
-    async def __call__(self, receive: Receive, send: Send) -> None:
-        request = self.context["request"]
-        extensions = request.get("extensions", {})
-        if "http.response.template" in extensions:
-            await send(
-                {
-                    "type": "http.response.template",
-                    "template": self.template,
-                    "context": self.context,
-                }
-            )
-        await super().__call__(receive, send)
-
-
 class JSONResponse(Response):
     media_type = "application/json"
 
@@ -193,10 +161,10 @@ class UJSONResponse(JSONResponse):
 
 class RedirectResponse(Response):
     def __init__(
-        self, url: typing.Union[str, URL], status_code: int = 302, headers: dict = None
+        self, url: typing.Union[str, URL], status_code: int = 307, headers: dict = None
     ) -> None:
         super().__init__(content=b"", status_code=status_code, headers=headers)
-        self.headers["location"] = quote_plus(str(url), safe=":/#?&=@[]!$&'()*+,;")
+        self.headers["location"] = quote_plus(str(url), safe=":/%#?&=@[]!$&'()*+,;")
 
 
 class StreamingResponse(Response):
@@ -208,13 +176,16 @@ class StreamingResponse(Response):
         media_type: str = None,
         background: BackgroundTask = None,
     ) -> None:
-        self.body_iterator = content
+        if inspect.isasyncgen(content):
+            self.body_iterator = content
+        else:
+            self.body_iterator = iterate_in_threadpool(content)
         self.status_code = status_code
         self.media_type = self.media_type if media_type is None else media_type
         self.background = background
         self.init_headers(headers)
 
-    async def __call__(self, receive: Receive, send: Send) -> None:
+    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
         await send(
             {
                 "type": "http.response.start",
@@ -238,6 +209,7 @@ class FileResponse(Response):
     def __init__(
         self,
         path: str,
+        status_code: int = 200,
         headers: dict = None,
         media_type: str = None,
         background: BackgroundTask = None,
@@ -247,7 +219,7 @@ class FileResponse(Response):
     ) -> None:
         assert aiofiles is not None, "'aiofiles' must be installed to use FileResponse"
         self.path = path
-        self.status_code = 200
+        self.status_code = status_code
         self.filename = filename
         self.send_header_only = method is not None and method.upper() == "HEAD"
         if media_type is None:
@@ -262,24 +234,17 @@ class FileResponse(Response):
         if stat_result is not None:
             self.set_stat_headers(stat_result)
 
-    @classmethod
-    def get_stat_headers(cls, stat_result: os.stat_result) -> typing.Dict[str, str]:
+    def set_stat_headers(self, stat_result: os.stat_result) -> None:
         content_length = str(stat_result.st_size)
         last_modified = formatdate(stat_result.st_mtime, usegmt=True)
         etag_base = str(stat_result.st_mtime) + "-" + str(stat_result.st_size)
         etag = hashlib.md5(etag_base.encode()).hexdigest()
-        return {
-            "content-length": content_length,
-            "last-modified": last_modified,
-            "etag": etag,
-        }
 
-    def set_stat_headers(self, stat_result: os.stat_result) -> None:
-        stat_headers = self.get_stat_headers(stat_result)
-        for name, value in stat_headers.items():
-            self.headers.setdefault(name, value)
+        self.headers.setdefault("content-length", content_length)
+        self.headers.setdefault("last-modified", last_modified)
+        self.headers.setdefault("etag", etag)
 
-    async def __call__(self, receive: Receive, send: Send) -> None:
+    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
         if self.stat_result is None:
             try:
                 stat_result = await aio_stat(self.path)
