@@ -10,7 +10,7 @@ from starlette.datastructures import URL, Headers, URLPath
 from starlette.exceptions import HTTPException
 from starlette.requests import Request
 from starlette.responses import PlainTextResponse, RedirectResponse
-from starlette.types import ASGIApp, ASGIInstance, Receive, Scope, Send
+from starlette.types import ASGIApp, Receive, Scope, Send
 from starlette.websockets import WebSocket, WebSocketClose
 
 
@@ -34,16 +34,13 @@ def request_response(func: typing.Callable) -> ASGIApp:
     """
     is_coroutine = asyncio.iscoroutinefunction(func)
 
-    def app(scope: Scope) -> ASGIInstance:
-        async def awaitable(receive: Receive, send: Send) -> None:
-            request = Request(scope, receive=receive)
-            if is_coroutine:
-                response = await func(request)
-            else:
-                response = await run_in_threadpool(func, request)
-            await response(receive, send)
-
-        return awaitable
+    async def app(scope: Scope, receive: Receive, send: Send) -> None:
+        request = Request(scope, receive=receive)
+        if is_coroutine:
+            response = await func(request)
+        else:
+            response = await run_in_threadpool(func, request)
+        await response(scope, receive, send)
 
     return app
 
@@ -54,12 +51,9 @@ def websocket_session(func: typing.Callable) -> ASGIApp:
     """
     # assert asyncio.iscoroutinefunction(func), "WebSocket endpoints must be async"
 
-    def app(scope: Scope) -> ASGIInstance:
-        async def awaitable(receive: Receive, send: Send) -> None:
-            session = WebSocket(scope, receive=receive, send=send)
-            await func(session)
-
-        return awaitable
+    async def app(scope: Scope, receive: Receive, send: Send) -> None:
+        session = WebSocket(scope, receive=receive, send=send)
+        await func(session)
 
     return app
 
@@ -135,7 +129,7 @@ class BaseRoute:
     def url_path_for(self, name: str, **path_params: str) -> URLPath:
         raise NotImplementedError()  # pragma: no cover
 
-    def __call__(self, scope: Scope) -> ASGIInstance:
+    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
         raise NotImplementedError()  # pragma: no cover
 
 
@@ -202,12 +196,15 @@ class Route(BaseRoute):
         assert not remaining_params
         return URLPath(path=path, protocol="http")
 
-    def __call__(self, scope: Scope) -> ASGIInstance:
+    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
         if self.methods and scope["method"] not in self.methods:
             if "app" in scope:
                 raise HTTPException(status_code=405)
-            return PlainTextResponse("Method Not Allowed", status_code=405)
-        return self.app(scope)
+            else:
+                response = PlainTextResponse("Method Not Allowed", status_code=405)
+            await response(scope, receive, send)
+        else:
+            await self.app(scope, receive, send)
 
     def __eq__(self, other: typing.Any) -> bool:
         return (
@@ -264,8 +261,8 @@ class WebSocketRoute(BaseRoute):
         assert not remaining_params
         return URLPath(path=path, protocol="websocket")
 
-    def __call__(self, scope: Scope) -> ASGIInstance:
-        return self.app(scope)
+    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
+        await self.app(scope, receive, send)
 
     def __eq__(self, other: typing.Any) -> bool:
         return (
@@ -276,13 +273,25 @@ class WebSocketRoute(BaseRoute):
 
 
 class Mount(BaseRoute):
-    def __init__(self, path: str, app: ASGIApp, name: str = None) -> None:
+    def __init__(
+        self,
+        path: str,
+        app: ASGIApp = None,
+        routes: typing.List[BaseRoute] = None,
+        name: str = None,
+    ) -> None:
         assert path == "" or path.startswith("/"), "Routed paths must start with '/'"
+        assert (
+            app is not None or routes is not None
+        ), "Either 'app=...', or 'routes=' must be specified"
         self.path = path.rstrip("/")
-        self.app = app
+        if app is not None:
+            self.app = app  # type: ASGIApp
+        else:
+            self.app = Router(routes=routes)
         self.name = name
         self.path_regex, self.path_format, self.param_convertors = compile_path(
-            path + "/{path:path}"
+            self.path + "/{path:path}"
         )
 
     @property
@@ -290,23 +299,24 @@ class Mount(BaseRoute):
         return getattr(self.app, "routes", None)
 
     def matches(self, scope: Scope) -> typing.Tuple[Match, Scope]:
-        path = scope["path"]
-        match = self.path_regex.match(path)
-        if match:
-            matched_params = match.groupdict()
-            for key, value in matched_params.items():
-                matched_params[key] = self.param_convertors[key].convert(value)
-            remaining_path = "/" + matched_params.pop("path")
-            matched_path = path[: -len(remaining_path)]
-            path_params = dict(scope.get("path_params", {}))
-            path_params.update(matched_params)
-            child_scope = {
-                "path_params": path_params,
-                "root_path": scope.get("root_path", "") + matched_path,
-                "path": remaining_path,
-                "endpoint": self.app,
-            }
-            return Match.FULL, child_scope
+        if scope["type"] in ("http", "websocket"):
+            path = scope["path"]
+            match = self.path_regex.match(path)
+            if match:
+                matched_params = match.groupdict()
+                for key, value in matched_params.items():
+                    matched_params[key] = self.param_convertors[key].convert(value)
+                remaining_path = "/" + matched_params.pop("path")
+                matched_path = path[: -len(remaining_path)]
+                path_params = dict(scope.get("path_params", {}))
+                path_params.update(matched_params)
+                child_scope = {
+                    "path_params": path_params,
+                    "root_path": scope.get("root_path", "") + matched_path,
+                    "path": remaining_path,
+                    "endpoint": self.app,
+                }
+                return Match.FULL, child_scope
         return Match.NONE, {}
 
     def url_path_for(self, name: str, **path_params: str) -> URLPath:
@@ -335,12 +345,12 @@ class Mount(BaseRoute):
                     return URLPath(
                         path=path.rstrip("/") + str(url), protocol=url.protocol
                     )
-                except NoMatchFound as exc:
+                except NoMatchFound:
                     pass
         raise NoMatchFound()
 
-    def __call__(self, scope: Scope) -> ASGIInstance:
-        return self.app(scope)
+    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
+        await self.app(scope, receive, send)
 
     def __eq__(self, other: typing.Any) -> bool:
         return (
@@ -362,17 +372,18 @@ class Host(BaseRoute):
         return getattr(self.app, "routes", None)
 
     def matches(self, scope: Scope) -> typing.Tuple[Match, Scope]:
-        headers = Headers(scope=scope)
-        host = headers.get("host", "").split(":")[0]
-        match = self.host_regex.match(host)
-        if match:
-            matched_params = match.groupdict()
-            for key, value in matched_params.items():
-                matched_params[key] = self.param_convertors[key].convert(value)
-            path_params = dict(scope.get("path_params", {}))
-            path_params.update(matched_params)
-            child_scope = {"path_params": path_params, "endpoint": self.app}
-            return Match.FULL, child_scope
+        if scope["type"] in ("http", "websocket"):
+            headers = Headers(scope=scope)
+            host = headers.get("host", "").split(":")[0]
+            match = self.host_regex.match(host)
+            if match:
+                matched_params = match.groupdict()
+                for key, value in matched_params.items():
+                    matched_params[key] = self.param_convertors[key].convert(value)
+                path_params = dict(scope.get("path_params", {}))
+                path_params.update(matched_params)
+                child_scope = {"path_params": path_params, "endpoint": self.app}
+                return Match.FULL, child_scope
         return Match.NONE, {}
 
     def url_path_for(self, name: str, **path_params: str) -> URLPath:
@@ -398,12 +409,12 @@ class Host(BaseRoute):
                 try:
                     url = route.url_path_for(remaining_name, **remaining_params)
                     return URLPath(path=str(url), protocol=url.protocol, host=host)
-                except NoMatchFound as exc:
+                except NoMatchFound:
                     pass
         raise NoMatchFound()
 
-    def __call__(self, scope: Scope) -> ASGIInstance:
-        return self.app(scope)
+    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
+        await self.app(scope, receive, send)
 
     def __eq__(self, other: typing.Any) -> bool:
         return (
@@ -413,6 +424,60 @@ class Host(BaseRoute):
         )
 
 
+class Lifespan(BaseRoute):
+    def __init__(
+        self, on_startup: typing.Callable = None, on_shutdown: typing.Callable = None
+    ):
+        self.startup_handlers = [] if on_startup is None else [on_startup]
+        self.shutdown_handlers = [] if on_shutdown is None else [on_shutdown]
+
+    def matches(self, scope: Scope) -> typing.Tuple[Match, Scope]:
+        if scope["type"] == "lifespan":
+            return Match.FULL, {}
+        return Match.NONE, {}
+
+    def add_event_handler(self, event_type: str, func: typing.Callable) -> None:
+        assert event_type in ("startup", "shutdown")
+
+        if event_type == "startup":
+            self.startup_handlers.append(func)
+        else:
+            assert event_type == "shutdown"
+            self.shutdown_handlers.append(func)
+
+    def on_event(self, event_type: str) -> typing.Callable:
+        def decorator(func: typing.Callable) -> typing.Callable:
+            self.add_event_handler(event_type, func)
+            return func
+
+        return decorator
+
+    async def startup(self) -> None:
+        for handler in self.startup_handlers:
+            if asyncio.iscoroutinefunction(handler):
+                await handler()
+            else:
+                handler()
+
+    async def shutdown(self) -> None:
+        for handler in self.shutdown_handlers:
+            if asyncio.iscoroutinefunction(handler):
+                await handler()
+            else:
+                handler()
+
+    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
+        message = await receive()
+        assert message["type"] == "lifespan.startup"
+        await self.startup()
+        await send({"type": "lifespan.startup.complete"})
+
+        message = await receive()
+        assert message["type"] == "lifespan.shutdown"
+        await self.shutdown()
+        await send({"type": "lifespan.shutdown.complete"})
+
+
 class Router:
     def __init__(
         self,
@@ -420,9 +485,10 @@ class Router:
         redirect_slashes: bool = True,
         default: ASGIApp = None,
     ) -> None:
-        self.routes = [] if routes is None else routes
+        self.routes = [] if routes is None else list(routes)
         self.redirect_slashes = redirect_slashes
         self.default = self.not_found if default is None else default
+        self.lifespan = Lifespan()
 
     def mount(self, path: str, app: ASGIApp, name: str = None) -> None:
         route = Mount(path, app=app, name=name)
@@ -481,31 +547,32 @@ class Router:
 
         return decorator
 
-    def not_found(self, scope: Scope) -> ASGIInstance:
+    async def not_found(self, scope: Scope, receive: Receive, send: Send) -> None:
         if scope["type"] == "websocket":
-            return WebSocketClose()
+            websocket_close = WebSocketClose()
+            await websocket_close(receive, send)
+            return
 
         # If we're running inside a starlette application then raise an
         # exception, so that the configurable exception handler can deal with
         # returning the response. For plain ASGI apps, just return the response.
         if "app" in scope:
             raise HTTPException(status_code=404)
-        return PlainTextResponse("Not Found", status_code=404)
+        else:
+            response = PlainTextResponse("Not Found", status_code=404)
+        await response(scope, receive, send)
 
     def url_path_for(self, name: str, **path_params: str) -> URLPath:
         for route in self.routes:
             try:
                 return route.url_path_for(name, **path_params)
-            except NoMatchFound as exc:
+            except NoMatchFound:
                 pass
         raise NoMatchFound()
 
-    def __call__(self, scope: Scope) -> ASGIInstance:
+    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
         assert scope["type"] in ("http", "websocket", "lifespan")
 
-        if scope["type"] == "lifespan":
-            return LifespanHandler(scope)
-
         if "router" not in scope:
             scope["router"] = self
 
@@ -515,40 +582,34 @@ class Router:
             match, child_scope = route.matches(scope)
             if match == Match.FULL:
                 scope.update(child_scope)
-                return route(scope)
+                await route(scope, receive, send)
+                return
             elif match == Match.PARTIAL and partial is None:
                 partial = route
                 partial_scope = child_scope
 
         if partial is not None:
             scope.update(partial_scope)
-            return partial(scope)
+            await partial(scope, receive, send)
+            return
 
-        if self.redirect_slashes and not scope["path"].endswith("/"):
-            redirect_scope = dict(scope)
-            redirect_scope["path"] += "/"
+        if scope["type"] == "http" and self.redirect_slashes:
+            if not scope["path"].endswith("/"):
+                redirect_scope = dict(scope)
+                redirect_scope["path"] += "/"
 
-            for route in self.routes:
-                match, child_scope = route.matches(redirect_scope)
-                if match != Match.NONE:
-                    redirect_url = URL(scope=redirect_scope)
-                    return RedirectResponse(url=str(redirect_url))
+                for route in self.routes:
+                    match, child_scope = route.matches(redirect_scope)
+                    if match != Match.NONE:
+                        redirect_url = URL(scope=redirect_scope)
+                        response = RedirectResponse(url=str(redirect_url))
+                        await response(scope, receive, send)
+                        return
 
-        return self.default(scope)
+        if scope["type"] == "lifespan":
+            await self.lifespan(scope, receive, send)
+        else:
+            await self.default(scope, receive, send)
 
     def __eq__(self, other: typing.Any) -> bool:
         return isinstance(other, Router) and self.routes == other.routes
-
-
-class LifespanHandler:
-    def __init__(self, scope: Scope) -> None:
-        pass
-
-    async def __call__(self, receive: Receive, send: Send) -> None:
-        message = await receive()
-        assert message["type"] == "lifespan.startup"
-        await send({"type": "lifespan.startup.complete"})
-
-        message = await receive()
-        assert message["type"] == "lifespan.shutdown"
-        await send({"type": "lifespan.shutdown.complete"})
